//
//  FlickrAPI.swift
//  Photorama
//
//  Created by Paul Pollock on 8/19/17.
//  Copyright © 2017 Paul Pollock. All rights reserved.
//

import Foundation

enum FlickrError: Error    {
    case invalidJSONData
}

enum Method: String {
    case interestingPhotos = "flickr.interestingness.getList"
}

struct FlickrAPI {

    private static let baseURLString = "https://api.flickr.com/services/rest"
    private static let apiKey = "a6d819499131071f158fd740860a5a88"
    
    private static let dateFormatter: DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateFormat = "yyyy-MM-dd HH:mm:ss"
        return formatter
    }()
    
    private static func photo(fromJSON json: [String : Any]) -> Photo?  {
        guard
        let photoID = json["id"] as? String,
        let title = json["title"] as? String,
        let dateString = json["dateTaken"] as? String,
        let photoURLString = json["url_h"] as? String,
        let url = URL(string: photoURLString),
        let dateTaken = dateFormatter.date(from: dateString)    else    {
//                print("Don't have enough information to construct a Photo") //  Appears, lots
                // Don't have enough information to construct a Photo
                return nil
        }
        return Photo(title: title, photoID: photoID, remoteURL: url, dateTaken: dateTaken)
    }

    private static func flickrURL(method: Method, parameters: [String:String]?) -> URL {
        
        var components = URLComponents(string: baseURLString)!
        
        var queryItems = [URLQueryItem]()

        let baseParams = [
            "method": method.rawValue,
            "format": "json",
            "nojsoncallback": "1",
            "api_key": apiKey
        ]
        
        for (key, value) in baseParams {
            let item = URLQueryItem(name: key, value: value)
            queryItems.append(item)
        }
        
        if let additionalParams = parameters {
            for (key,value) in additionalParams {
                let item = URLQueryItem(name: key, value: value)
                queryItems.append(item)
            }
        }

        components.queryItems = queryItems
        
//        return URL(string: "")
        
        return components.url!
   }
    
    static var interestingPhotosURL: URL {
        return flickrURL(method: .interestingPhotos, parameters: ["extras": "url_h,date_taken"])
    }

    static func photos(fromJSON data: Data) -> PhotosResult {
        do {
            let jsonObject = try JSONSerialization.jsonObject(with: data, options: [])
//            print("jsonObject: \(jsonObject)")  //  Lots, nicely organized
            guard
                let jsonDictionary = jsonObject as? [AnyHashable:Any],
                let photos = jsonDictionary["photos"] as? [String:Any],
                let photosArray = photos["photo"] as? [[String:Any]]

                
                else {
//                    print("static func photos (else FlickrAPI.swift")   //  Does not appear
                    // The JSON structure doesn't match our expectations
                    return .failure(FlickrError.invalidJSONData)
            }

            var finalPhotos = [Photo]()
//            print("photosArray1: \(photosArray)")    //  Lots, not nicely organized
            for photoJSON in photosArray {
                if let photo = photo(fromJSON: photoJSON)   {
                    print("if let photo = photo(fromJSON: photoJSON)")  //  not there
                    finalPhotos.append(photo)
                }
            }
//            print("photosArray2: \(photosArray)")    //  lots, not nicely organized
//            print("finalPhotos: \(finalPhotos)")    //  0 values
            if finalPhotos.isEmpty && !photosArray.isEmpty  {   //  page 596
//                print("if finalPhotos.isEmpty && !photosArray.isEmpty") //  Appears
                // We weren't able to parse any of the photos
                // Maybe the JSON format for photos has changed
                return .failure(FlickrError.invalidJSONData)
            }
            print("return .success(finalPhotos)")   //  Does not appear
                return .success(finalPhotos)
            } catch let error {
                print("return .failure(error)") //  Does not appear
                return .failure(error)
        }
    }
}
//}
